/*
Copyright (c) 1993, Clemens Wagner
 
Permission to use, copy, modify, and distribute this software and 
its documentation for any purpose is hereby granted without fee, provided
that
(i) you have alays to distribute the whole archive
(ii) the above copyright notices and this permission notice appear in
all copies of the software and related documentation, changed or new
code has to be marked
(iii) else, there are the same conditions as for InterViews

THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 */

#include <Graphic/text.h>
#include <Graphic/handles.h>

#include <InterViews/brush.h>
#include <InterViews/canvas.h>
#include <InterViews/character.h>
#include <InterViews/color.h>
#include <InterViews/font.h>
#include <InterViews/layout.h>
#include <InterViews/tformsetter.h>
#include <InterViews/transformer.h>
#include <IV-X11/xcanvas.h>
#include <OS/string.h>
#include <stdlib.h>
#include <stream.h>
#include <string.h>
#include <ctype.h>

inline char *strnew(const char *s)
{
	char *n=new char[strlen(s)+1];

	strcpy(n,s);
	return n;
}

Text::Text(const Font *f,const Color *fg,const char *text,Transformer *tx)
{
	LayoutKit *layout=LayoutKit::instance();

	_stroke=fg;
	Resource::ref(_stroke);
	_font=f;
	Resource::ref(_font);
	_t=new Transformer;
	if(tx!=nil)
		*_t=*tx;
	_fill=fg; /*a hack */
	if(text!=nil)
		_text=new CopyString(text);
	else
		_text=new CopyString("");
	compute_metrics();
}

Text::~Text()
{
	delete _text;
}

Glyph *Text::clone() const
{
	return new Text(_font,_stroke,_text->string(),_t);
}

void Text::text(const char *text)
{
	delete _text;
	if(text!=nil)
		_text=new CopyString(text);
	else
		_text=new CopyString("");
	compute_metrics();
}

const char *Text::text()
{
	return _text->string();
}

void Text::draw(Canvas *c,const Allocation &a) const
{
	Graphic::draw(c,a);
}

void Text::allocate(Canvas *c,const Allocation &a,Extension &ext)
{
	if(_t!=nil)
	{
		c->push_transform();
		c->transform(*_t);
	}
	if(_ctrlpts==0)
	{
		Requisition req;
		request(req);
	}
	_a=a;
	Allocation b(_a);
	Allotment &bx=b.x_allotment();
	Allotment &by=b.y_allotment();
	bx.origin(0.0);
	by.origin(0.0);
	bx.span(_x[2]-_x[0]);
	by.span(_y[2]-_y[0]);
	if(_t!=nil)
		c->pop_transform();
}

void Text::request(Requisition &req) const
{ 
	Text *text=(Text *) this;
	Coord left,bottom,right,top;
	Coord height=_ascent+_descent;
	float alignment=(height==0) ? 0 : _descent/height;
	Requirement &rx=req.x_requirement();
	Requirement &ry=req.y_requirement();

	left=0;
	right=_width;
	bottom=-height*alignment;
	top=bottom+height;

	text->_ctrlpts=4;
	text->_x[0]=left;
	text->_y[0]=bottom;
	text->_x[1]=left;
	text->_y[1]=top;
	text->_x[2]=right;
	text->_y[2]=top;
	text->_x[3]=right;
	text->_y[3]=bottom;

	if(_t!=nil)
		corners(left,bottom,right,top,*_t);
	rx.natural(right-left);
	rx.stretch(0.0);
	rx.shrink(0.0);
	rx.alignment(0);

	ry.natural(top-bottom);
	ry.stretch(0.0);
	ry.shrink(0.0);
	ry.alignment(-bottom/ry.natural());
}

ClassId Text::get_classid() const
{
	return TEXT_GRAPHIC;
}

boolean Text::is_a(ClassId id) const
{
	return id==TEXT_GRAPHIC || Graphic::is_a(id);
}

//

void Text::getextent_gs(Coord &l,Coord &b,Coord &cx,Coord &cy,float &tol,Graphic *gr)
{
	Coord left,bottom,right,top;

	if(_ctrlpts==0)
	{
		Requisition req;
		request(req);
	}
	right=_x[2];
	top=_y[2];
	left=_x[0];
	bottom=_y[0];

	Transformer *t=gr->transformer();
	if(t!=nil)
		corners(left,bottom,right,top,*t);
	tol=3.0;
	l=left;
	b=bottom;
	cx=(left+right)/2.0;
	cy=(top+bottom)/2.0;
}

void Text::draw_gs(Canvas *c,Graphic *gs)
{
	Transformer *tx=gs->transformer();
	Coord x=_a.x();
	Coord y=_a.y();
	const char *p=_text->string();
	const char *q=p+_text->length();
	Coord *cw=_char_widths;

	c->push_transform();
	if(tx!=nil)
		c->transform(*tx);
	while(p<q)
	{
		c->character(_font,*p,*cw,_stroke,x,y);
		x+=*cw;
		p++;
		cw++;
	}
	/*
	if(_handles!=nil)
	{
		c->transformer(nil);
		create_handles();
		if(_handles->drawn())
			_handles->redraw();
		else
			_handles->draw(c);
	}
	*/
	c->pop_transform();
}

void Text::compute_metrics()
{
	const Font *f=_font;
	const char *str =_text->string();
	int len=_text->length();
	FontBoundingBox b;
	int i;

	f->string_bbox(str,len,b);
	_ascent=b.font_ascent();
	_descent=b.font_descent();
	_left=b.left_bearing();
	_right=b.right_bearing();
	_width=b.width();
	_char_widths=new Coord[len];
	for(i=0;i<len;i++)
		_char_widths[i] = f->width(((u_char*) str)[i]);
}
