#include <Graphic/connector.h>
#include <Graphic/conntool.h>
#include <Graphic/cutcmd.h>
#include <Graphic/ellipses.h>
#include <Graphic/glyphviewer.h>
#include <Graphic/graphicmaster.h>
#include <Graphic/graphictool.h>
#include <Graphic/link.h>
#include <Graphic/menutool.h>
#include <Graphic/movetool.h>
#include <Graphic/polygraphic.h>
#include <Graphic/tool_button.h>

#include "lineconnect.h"

#include <IV-look/dialogs.h>
#include <IV-look/field.h>
#include <IV-look/fchooser.h>
#include <IV-look/kit.h>
#include <IV-look/slider.h>
#include <IV-look/telltale.h>

#include <InterViews/adjust.h>
#include <InterViews/background.h>
#include <InterViews/brush.h>
#include <InterViews/color.h>
#include <InterViews/layout.h>
#include <InterViews/patch.h>
#include <InterViews/session.h>
#include <InterViews/style.h>
#include <InterViews/transformer.h>
#include <InterViews/window.h>
#include <OS/string.h>
#include <stdio.h>
#include <stream.h>

class InfoMenu: public ToolMenu
{
	public:
		InfoMenu(Menu *);
		virtual ~InfoMenu() {Resource::unref(_mdialog);};

		void black() { target()->fill(new Color(0,0,0,1)); }
		void white() { target()->fill(new Color(1.0,1.0,1.0,1)); }
		void green() { target()->fill(new Color(0,1.0,0,1)); }
		void blue()  { target()->fill(new Color(0,0,1.0,1)); }
		void red()   { target()->fill(new Color(1.0,0,0,1)); }

		void zoom_in();
		void zoom_out();

		void move();
		void duplicate();
		void ok();
		void cancel();

		void file_select();

		Dialog *move_dialog();
	private:
		Dialog *_mdialog;
};

InfoMenu::InfoMenu(Menu *m): ToolMenu(m)
{
	_mdialog=move_dialog();
	Resource::ref(_mdialog);
}

void InfoMenu::zoom_in()
{
	Coord x,y;

	target()->getcenter(x,y);
	target()->scale(0.5,0.5,x,y);
}

void InfoMenu::zoom_out()
{
	Coord x,y;

	target()->getcenter(x,y);
	target()->scale(2,2,x,y);
}

void InfoMenu::move()
{
	_mdialog->post_for(ToolMenu::main_window());
}

void InfoMenu::duplicate()
{
	Graphic *g=(Graphic *) target()->clone();
	GraphicMaster *gm=(GraphicMaster *) target()->root();
	Coord l,b,r,t;

	g->getbounds(l,b,r,t);
	g->translate((r-l)/2,(t-b)/2);
	gm->append_(g);
}

void InfoMenu::ok()
{
	_mdialog->dismiss(true);
}

void InfoMenu::cancel()
{
	_mdialog->dismiss(false);
}

void InfoMenu::file_select()
{
	WidgetKit *k=WidgetKit::instance();
	FileChooser *f=new FileChooser(String("."),k,k->style());

	f->post_for(ToolMenu::main_window());
}

declareActionCallback(InfoMenu)
implementActionCallback(InfoMenu)

Dialog *InfoMenu::move_dialog()
{
	DialogKit &d=*DialogKit::instance();
	WidgetKit &k=*WidgetKit::instance();
	LayoutKit &l=*LayoutKit::instance();
	PolyGlyph *vbox=l.vbox();
	PolyGlyph *hbox=l.hbox();
	FieldEditor *fc;
	Button *b;
	Dialog *dialog=new Dialog(k.inset_frame(l.margin(vbox,16)),k.style());

	vbox->append(k.outset_frame(k.label("Please enter movement: ")));
	vbox->append(l.vglue(16,0,8));
	fc=d.field_editor("",k.style());
	vbox->append(fc);
	dialog->append_input_handler(fc);
	vbox->append(l.vglue(8,0,4));
	fc=d.field_editor("",k.style());
	vbox->append(fc);
	dialog->append_input_handler(fc);
	vbox->append(l.vglue(16,0,8));
	hbox->append(l.hglue(0,fil,0));
	hbox->append(k.push_button("Cancel",new ActionCallback(InfoMenu)(this,&InfoMenu::cancel)));
	hbox->append(l.hglue(4,0,2));
	hbox->append(b=k.default_button("  OK  ",new ActionCallback(InfoMenu)(this,&InfoMenu::ok)));
	dialog->append_input_handler(b);
	vbox->append(hbox);
	return dialog;
}

InfoMenu *pulldown()
{
	WidgetKit &k=*WidgetKit::instance();
	LayoutKit &l=*LayoutKit::instance();
	Menu *m=k.pulldown();
	Menu *pr1=k.pullright();
	Menu *pr2=k.pullright();
	MenuItem *mi;
	Glyph *g;
	InfoMenu *im=new InfoMenu(m);

	g=l.rmargin(k.fancy_label("File"),0.0,fil,0.0);
	mi=k.menu_item(g);
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::file_select));
	m->append_item(mi);

	g=l.hbox(k.fancy_label("Color"),l.hglue(0,fil,0),k.fancy_label(">"));
	mi=k.menu_item(g);
	m->append_item(mi);
	mi->menu(pr1);
	g=l.rmargin(k.fancy_label("Black"),0.0,fil,0.0);
	mi=k.menu_item(g);
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::black));
	pr1->append_item(mi);
	g=l.rmargin(k.fancy_label("White"),0.0,fil,0.0);
	mi=k.menu_item(g);
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::white));
	pr1->append_item(mi);
	g=l.rmargin(k.fancy_label("Green"),0.0,fil,0.0);
	mi=k.menu_item(g);
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::green));
	pr1->append_item(mi);
	g=l.rmargin(k.fancy_label("Blue"),0.0,fil,0.0);
	mi=k.menu_item(g);
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::blue));
	pr1->append_item(mi);
	g=l.rmargin(k.fancy_label("Red"),0.0,fil,0.0);
	mi=k.menu_item(g);
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::red));
	pr1->append_item(mi);

	m->append_item(k.menu_item_separator());

	g=l.hbox(k.fancy_label("Scaling"),l.hglue(0,fil,0),k.fancy_label(">"));
	mi=k.menu_item(g);
	m->append_item(mi);
	mi->menu(pr2);
	g=l.rmargin(k.fancy_label("Enlarge "),0.0,fil,0.0);
	mi=k.menu_item(g);
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::zoom_out));
	pr2->append_item(mi);
	g=l.rmargin(k.fancy_label("Reduce "),0.0,fil,0.0);
	mi=k.menu_item(g);
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::zoom_in));
	pr2->append_item(mi);

	g=l.rmargin(k.fancy_label("Move..."),0.0,fil,0.0);
	mi=k.menu_item(g);
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::move));
	m->append_item(mi);
	g=l.rmargin(k.fancy_label("Duplicate"),0.0,fil,0.0);
	mi=k.menu_item(g);
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::duplicate));
	m->append_item(mi);

	m->append_item(k.menu_item_separator());

	g=l.rmargin(k.fancy_label("Quit"),0.0,fil,0.0);
	mi=k.menu_item(g);
	mi->action(k.quit());
	m->append_item(mi);
	return im;
}
	
Glyph *interior(GraphicMaster *gm,InfoMenu *im)
{
	Menu *pm,*mb,*pr;
	MenuItem *mi;
	WidgetKit& kit = *WidgetKit::instance();
	const LayoutKit& layout = *LayoutKit::instance();

	mb=kit.menubar();

	pm=kit.pulldown();
	mi=kit.menu_item(layout.r_margin(kit.fancy_label("Open...  "),0.0,fil,0.0));
	pm->append_item(mi);
	pm->append_item(kit.menu_item_separator());
	mi=kit.menu_item(layout.r_margin(kit.fancy_label("Save  "),0.0,fil,0.0));
	mi->state()->set(TelltaleState::is_enabled,false);
	pm->append_item(mi);
	mi=kit.menu_item(layout.r_margin(kit.fancy_label("Save as...  "),0.0,fil,0.0));
	pm->append_item(mi);
	pm->append_item(kit.menu_item_separator());
	mi=kit.menu_item(layout.r_margin(kit.fancy_label("Quit  "),0.0,fil,0.0));
	pm->append_item(mi);
	mi->action(kit.quit());
	mi=kit.menubar_item(kit.fancy_label("File"));
	mi->menu(pm);
	mb->append_item(mi);

	pm=kit.pulldown();
	mi=kit.menu_item(layout.r_margin(kit.fancy_label("Cut"),0.0,fil,0.0));
	mi->action(new CommandAction(new CutCmd(gm)));
	pm->append_item(mi);
	mi=kit.menu_item(layout.r_margin(kit.fancy_label("Copy"),0.0,fil,0.0));
	pm->append_item(mi);
	mi=kit.menu_item(layout.r_margin(kit.fancy_label("Paste"),0.0,fil,0.0));
	pm->append_item(mi);
	pm->append_item(kit.menu_item_separator());
	mi=kit.menu_item(kit.fancy_label("Move"));
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::move));
	pm->append_item(mi);
	mi=kit.menubar_item(kit.fancy_label("Edit"));
	mi->menu(pm);
	mb->append_item(mi);
	mi=kit.menu_item(kit.fancy_label("Move"));
	mi->action(new ActionCallback(InfoMenu)(im,&InfoMenu::move));

	pm=kit.pulldown();
	mi=kit.menubar_item(layout.r_margin(kit.fancy_label("Size  "),0.0,fil,0.0));
	pm->append_item(mi);
	pr=kit.pullright();
	mi->menu(pr);
	mi=kit.menu_item(layout.r_margin(kit.fancy_label("10 pt  "),0.0,fil,0.0));
	pr->append_item(mi);
	mi=kit.menu_item(layout.r_margin(kit.fancy_label("11 pt  "),0.0,fil,0.0));
	pr->append_item(mi);
	mi=kit.menu_item(layout.r_margin(kit.fancy_label("12 pt  "),0.0,fil,0.0));
	pr->append_item(mi);
	mi=kit.menubar_item(kit.fancy_label("Font"));
	mi->menu(pm);
	mb->append_item(mi);
	mi=kit.menubar_item(layout.r_margin(kit.fancy_label("Glibber"),0.0,fil,0.0));
	pm->append_item(mi);
	mi=kit.menubar_item(layout.r_margin(kit.fancy_label("Moder"),0.0,fil,0.0));
	pm->append_item(mi);

	return kit.outset_frame(mb);
}

Graphic *ball(Coord rad,float rl,float rh,float gl,float gh,float bl,float bh)
{
	PolyGraphic *pg=new PolyGraphic;
	Graphic *g=new FixedConnector(nil,nil,nil,nil);
	Coord x,y;
	float rs=2*(rh-rl)/rad;
	float gs=2*(gh-gl)/rad;
	float bs=2*(bh-bl)/rad;
	int i;

	pg->append_(g);
	g->translate(rad,rad);
	for(i=rad;i>0;i-=2)
	{
		pg->append_(new Circle(nil,nil,new Color(rl,gl,bl,1),rad,rad,i,nil));
		rl+=rs;
		gl+=gs;
		bl+=bs;
	}
	return pg;
}

int main(int argc,char **argv)
{
	Session *session=new Session("Himom",argc,argv);
	WidgetKit &kit=*WidgetKit::instance();
	const LayoutKit &layout=*LayoutKit::instance();
	Style *style=session->style();
	GlyphViewer *gv=new GlyphViewer(300,300,8.0*72.0,12.0*72.0);
	GraphicMaster *g=gv->root();
	Color *black=new Color(0,0,0,1);
	Glyph *gl;
	PolyGlyph *vbox=layout.vbox();
	PolyGlyph *hbox=layout.hbox_first_aligned();
	Tool *t=new MoveTool(g);
	TelltaleState *ts;
	TelltaleGroup *tg=new TelltaleGroup;
	TelltaleFlags flags=TelltaleState::is_enabled|TelltaleState::is_choosable;
	InfoMenu *im=pulldown();
	Button *menu=tool_button(kit.label("Menu"),flags,tg,new ToolAction(new MenuTool(g,im)));
	Button *move=tool_button(kit.label("Move"),flags,tg,new ToolAction(t));
	Button *bb0=tool_button
	(
		layout.hmargin(new GraphicGlyph(ball(15,0,1,0,1,0,1)),0,fil,0,0,fil,0),
		flags,tg,new ToolAction(new GraphicTool(g,ball(60,0,1,0,1,0,1)))
	);
	Button *bb1=tool_button
	(
		layout.hmargin(new GraphicGlyph(ball(15,0.1,1,0,0,0,0)),0,fil,0,0,fil,0),
		flags,tg,new ToolAction(new GraphicTool(g,ball(60,0.1,1,0,0,0,0)))
	);
	Button *bb2=tool_button
	(
		layout.hmargin(new GraphicGlyph(ball(15,0.0,0.0,0.1,1,0.0,0.0)),0,fil,0,0,fil,0),
		flags,tg,new ToolAction(new GraphicTool(g,ball(60,0.0,0.0,0.1,1,0.0,0.0)))
	);
	Button *bb3=tool_button
	(
		layout.hmargin(new GraphicGlyph(ball(15,0,0,0,0,0.1,1)),0,fil,0,0,fil,0),
		flags,tg,new ToolAction(new GraphicTool(g,ball(60,0,0,0,0,0.1,1)))
	);
	Button *bb4=tool_button
	(
		layout.hmargin(new GraphicGlyph(ball(15,0.1,1,0.1,1,0,0)),0,fil,0,0,fil,0),
		flags,tg,new ToolAction(new GraphicTool(g,ball(60,0.1,1,0.1,1,0,0)))
	);
	Button *bb5=tool_button
	(
		layout.hmargin(new GraphicGlyph(ball(15,0,1,0,0,0,1)),0,fil,0,0,fil,0),
		flags,tg,new ToolAction(new GraphicTool(g,ball(60,0.1,1,0,0,0.1,1)))
	);
	Button *bb6=tool_button
	(
		layout.hmargin(new GraphicGlyph(ball(15,0,0,0,1,0,1)),0,fil,0,0,fil,0),
		flags,tg,new ToolAction(new GraphicTool(g,ball(60,0,0,0.1,1,0.1,1)))
	);
	Button *connect=tool_button
	(
		kit.label("Connect"),flags,tg,
		new ToolAction(new LineConnectTool(g,new Link(nil,nil,new Brush(3),black,new Color(1.0,0,0,1.0))))
	);
	Slider *panner;

	ts=move->state();
	ts->set(TelltaleState::is_active,true);
	g->background(new Color(1,1,1,1));
	gv->tool(t);
	vbox->append(menu);
	vbox->append(move);
	vbox->append(bb0);
	vbox->append(bb1);
	vbox->append(bb2);
	vbox->append(bb3);
	vbox->append(bb4);
	vbox->append(bb5);
	vbox->append(bb6);
	vbox->append(connect);
	//vbox->append(kit.inset_frame(layout.hmargin(layout.hbox(kit.enlarger(gv),kit.reducer(gv)),0,fil,0,0,fil,0)));
	panner=(Slider *) kit.panner(gv,gv);
	gv->panner(panner);
	vbox->append(kit.inset_frame(panner));
	vbox->append(kit.inset_frame(kit.hslider(gv->zoomer())));
	hbox->append(layout.vcenter(kit.outset_frame(layout.vmargin(vbox,0,fil,0,0,fil,0)),0.0));
	//hbox->append(layout.vbox(kit.inset_frame(gv)));
	hbox->append(kit.inset_frame(gv));
	gl=layout.vbox(kit.inset_frame(interior(g,im)),hbox);
	ToolMenu::main_window(new ApplicationWindow(gl));
	ToolMenu::main_window()->style(style);
	return session->run_window(ToolMenu::main_window());
}

